package schelling;

/*-------------------------------------------------------------------------------

File:			AgentStyle2D.java
Version:		1
Package:		schelling
Class:			AgentStyle2D

Author:			Alan Mills

Description:	This file contains the class 'AgentStyle2D' for the Schelling Segregation model.
				The model supports Chapter seven of the report
				"Complexity Science - an introduction (and invitation) for actuaries"
				written in 2010 for the SOA by Alan Mills.

Methods:    	In addition to a constructor, the file contains the following methods:
				1. getPNode(Agent agent, PNode node)
				2. getPaint(Agent agent)
              	3. instance variable methods ("getters" and "setters")

Notes:        	1.	See "Chapter six:  Artificial societies" of "Complexiy Science -
                 	an introduction (and invitation) for actuaries" to learn more
                 	about the structure of this program and about agent-based
                 	modeling in general.
              	2. 	Even though this program is relatively elementary, it is useful
                 	to review, because it employs certain agent-based programming
                 	techniques that are used for more complicated Complexity
                 	Science models.

-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
TO DO
1.  
------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
IMPORTED FILES
-------------------------------------------------------------------------------*/
import java.awt.Color;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import repast.simphony.visualization.visualization2D.style.Style2D;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.nodes.PText;

/*-------------------------------------------------
 Because this class implements the Simphony class
 Style2D, it must also implement the 'getters' of
 Style2D<Agent>, such as getPNode(), etc.
---------------------------------------------------*/		
public class AgentStyle2D implements Style2D<Agent>
{
	/*-------------------------------------------------
	 Declare class instance variables
	 These variables correspond to the instance variables
	 in Style2D<Agent>. They must all be declared, even
	 though the last three aren't used.
	---------------------------------------------------*/		
	private Rectangle2D agentRectangle;
	private Shape agentShape;
	private Stroke agentStroke;
	private Paint agentStrokePaint;	
	private double agentRotation;
	
	// The class constructor. This is called by the Simphony runtime environment.
	public AgentStyle2D()
	{
		super();
		
		agentStroke = null;
		agentStrokePaint = null;
		agentRotation = 0.0;
		// The rectangle defines the boundary of each agent's disk.
		agentRectangle = new Rectangle2D.Float(0, 0, 10, 10);
		// This defines the disk's shape as a circle.
		agentShape = new Ellipse2D.Float(0,0,10,10);
		
	}// end AgentStyle2D() constructor

	/*=============================================================================
	 Method:     	1. getPNode(Agent agent, PNode node)
	 Description:  	Sets how a node is initially drawn.
	 ------------------------------------------------------------------------------*/
	public PNode getPNode(Agent agent, PNode node)
	{
		PPath agentPath;
		
		agentPath = new PPath(agentShape);
		agentPath.setBounds(agentRectangle);
		agentPath.setPaint(this.getPaint(agent));
		agentPath.setStroke(this.getStroke(agent));
		agentPath.setStrokePaint(this.getStrokePaint(agent));

		return agentPath;
	}
	
	/*=============================================================================
	 Method:     	2. getPaint(Agent agent)
	 Description:  	Sets a node's color.
	 ------------------------------------------------------------------------------*/
	public Paint getPaint(Agent agent)
	{
		if (agent.getGroupID() == 1)	
			return Color.BLUE;
		else if ((agent.getGroupID() == 0))
			return Color.RED;
		
		return null;
	}

	/*=============================================================================
	 Method:     	3. Instance return methods ('getters' and 'setters')
	 Description:  	Methods to return instance variables.
	 ------------------------------------------------------------------------------*/	
	public Stroke getStroke(Agent agent)
	{
		return agentStroke;
	}
	
	public Paint getStrokePaint(Agent agent)
	{
		return agentStrokePaint;
	}	
	
	public double getRotation(Agent agent)
	{
		return agentRotation;
	}
	
	public void setRotation(double inputRotation)
	{
		this.agentRotation = inputRotation;
	}
	
	public Rectangle2D getBounds(Agent agent)
	{
		return agentRectangle;
	}
	
	public void setBounds(Rectangle2D bounds)
	{
		this.agentRectangle = bounds;
	}

	public boolean isScaled(Agent agent)
	{
		return true;
	}

	public PText getLabel(Agent agent)
	{
		return null;
	}
	
}// end class AgentStyle2D
